@@ -6,14 +6,15 @@
 */
 
 #include "unversionedplugin.h"
+#include "kcoreaddons_debug.h"
+#include <QDebug>
 #include <kexportplugin.h>
 #include <kpluginfactory.h>
-#include <QDebug>
 
 UnversionedPlugin::UnversionedPlugin(QObject *parent, const QVariantList &args)
     : QObject(parent)
 {
-    qDebug() << "Created UnversionedPlugin with args" << args;
+    qCDebug(KCOREADDONS_DEBUG) << "Created UnversionedPlugin with args" << args;
 }
 
 K_PLUGIN_FACTORY(UnversionedPluginFactory, registerPlugin<UnversionedPlugin>();)
