@@ -1,30 +1,20 @@
-/*  This file is part of the KDE project
-    Copyright (C) 2000 Stephan Kulow <coolo@kde.org>
-                       David Faure <faure@kde.org>
-    Copyright (C) 2006 Kevin Ottens <ervin@kde.org>
+/*
+    This file is part of the KDE project
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License version 2 as published by the Free Software Foundation.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-FileCopyrightText: 2000 Stephan Kulow <coolo@kde.org>
+    SPDX-FileCopyrightText: 2000 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 2006 Kevin Ottens <ervin@kde.org>
 
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #ifndef KJOB_H
 #define KJOB_H
 
-#include <kcoreaddons_export.h>
 #include <QObject>
 #include <QPair>
+#include <kcoreaddons_export.h>
+#include <memory>
 
 class KJobUiDelegate;
 
@@ -39,8 +29,8 @@ class KJobPrivate;
  * void SomeClass::methodWithAsynchronousJobCall()
  * {
  *   KJob* job = someoperation(some parameters);
- *   connect(job, SIGNAL(result(KJob*)),
- *           this, SLOT(handleResult(KJob*)));
+ *   connect(job, &KJob::result,
+ *           this, &SomeClass::handleResult);
  *   job->start();
  * }
  * \endcode
@@ -49,7 +39,7 @@ class KJobPrivate;
  * And handleResult is usually at least:
  *
  * \code
- * void SomeClass::handleResult( KJob *job )
+ * void SomeClass::handleResult(KJob *job)
  * {
  *   if (job->error()) {
  *       doSomething();
@@ -89,19 +79,38 @@ class KCOREADDONS_EXPORT KJob : public QObject
     Q_PROPERTY(int error READ error NOTIFY result)
     Q_PROPERTY(QString errorText READ errorText NOTIFY result)
     Q_PROPERTY(QString errorString READ errorString NOTIFY result)
-    Q_PROPERTY(unsigned long percent READ percent NOTIFY percent)
+    Q_PROPERTY(ulong percent READ percent NOTIFY percent) // KF6 TODO: make "int", is enough
     Q_PROPERTY(Capabilities capabilities READ capabilities CONSTANT)
 
 public:
-    enum Unit { Bytes, Files, Directories };
+    /**
+     * Describes the unit used in the methods that handle reporting the job progress info.
+     * @see totalAmount
+     */
+    enum Unit {
+        Bytes = 0, ///< Directory and file sizes in bytes
+        Files, ///< The number of files handled by the job
+        Directories, ///< The number of directories handled by the job
+        Items, ///< The number of items (e.g. both directories and files) handled by the job
+               ///< @since 5.72
+
+        UnitsCount, ///< @internal since 5.87, used internally only, do not use.
+    };
     Q_ENUM(Unit)
 
-    enum Capability { NoCapabilities = 0x0000,
-                      Killable       = 0x0001,
-                      Suspendable    = 0x0002
-                    };
+    /**
+     * @see Capabilities
+     */
+    enum Capability {
+        NoCapabilities = 0x0000, ///< None of the capabilities exist
+        Killable = 0x0001, ///< The job can be killed
+        Suspendable = 0x0002, ///< The job can be suspended
+    };
     Q_ENUM(Capability)
 
+    /**
+     * Stores a combination of #Capability values.
+     */
     Q_DECLARE_FLAGS(Capabilities, Capability)
     Q_FLAG(Capabilities)
 
@@ -168,13 +177,16 @@ public:
      * \code
      * void ExampleJob::start()
      * {
-     *  QTimer::singleShot(0, this, SLOT(doWork()));
+     *  QTimer::singleShot(0, this, &ExampleJob::doWork);
      * }
      * \endcode
      */
     Q_SCRIPTABLE virtual void start() = 0;
 
-    enum KillVerbosity { Quietly, EmitResult };
+    enum KillVerbosity {
+        Quietly,
+        EmitResult,
+    };
     Q_ENUM(KillVerbosity)
 
 public Q_SLOTS:
@@ -191,6 +203,7 @@ public Q_SLOTS:
      * set to Quietly, signal result will NOT be emitted.
      * @return true if the operation is supported and succeeded, false otherwise
      */
+    // TODO KF6 slot arguments need to be fully-qualified, KJob::KillVerbosity
     bool kill(KillVerbosity verbosity = Quietly);
 
     /**
@@ -267,7 +280,7 @@ public:
         /*** Indicates the job was killed */
         KilledJobError = 1,
         /*** Subclasses should define error codes starting at this value */
-        UserDefinedError = 100
+        UserDefinedError = 100,
     };
 
     /**
@@ -359,6 +372,8 @@ Q_SIGNALS:
      * Emitted when the job is finished, in any case. It is used to notify
      * observers that the job is terminated and that progress can be hidden.
      *
+     * @since 5.75: this signal is guaranteed to be emitted exactly once.
+     *
      * This is a private signal, it can't be emitted directly by subclasses of
      * KJob, use emitResult() instead.
      *
@@ -374,9 +389,10 @@ Q_SIGNALS:
      */
     void finished(KJob *job
 #if !defined(K_DOXYGEN)
-                  , QPrivateSignal
+                  ,
+                  QPrivateSignal
 #endif
-                 );
+    );
 
     /**
      * Emitted when the job is suspended.
@@ -388,9 +404,10 @@ Q_SIGNALS:
      */
     void suspended(KJob *job
 #if !defined(K_DOXYGEN)
-                   , QPrivateSignal
+                   ,
+                   QPrivateSignal
 #endif
-                  );
+    );
 
     /**
      * Emitted when the job is resumed.
@@ -402,13 +419,16 @@ Q_SIGNALS:
      */
     void resumed(KJob *job
 #if !defined(K_DOXYGEN)
-                 , QPrivateSignal
+                 ,
+                 QPrivateSignal
 #endif
-                );
+    );
 
     /**
      * Emitted when the job is finished (except when killed with KJob::Quietly).
      *
+     * @since 5.75: this signal is guaranteed to be emitted at most once.
+     *
      * Use error to know if the job was finished with error.
      *
      * This is a private signal, it can't be emitted directly by subclasses of
@@ -422,9 +442,10 @@ Q_SIGNALS:
      */
     void result(KJob *job
 #if !defined(K_DOXYGEN)
-                , QPrivateSignal
+                ,
+                QPrivateSignal
 #endif
-               );
+    );
 
     /**
      * Emitted to display general description of this job. A description has
@@ -439,7 +460,8 @@ Q_SIGNALS:
      * @param field1 first field (localized name and value)
      * @param field2 second field (localized name and value)
      */
-    void description(KJob *job, const QString &title,
+    void description(KJob *job,
+                     const QString &title,
                      const QPair<QString, QString> &field1 = QPair<QString, QString>(),
                      const QPair<QString, QString> &field2 = QPair<QString, QString>());
     /**
@@ -448,9 +470,9 @@ Q_SIGNALS:
      *
      * @param job the job that emitted this signal
      * @param plain the info message
-     * @param rich the rich text version of the message, or QString() is none is available
+     * @param rich the rich text version of the message, or QString() is none is available -- do not use, it's ignored
      */
-    void infoMessage(KJob *job, const QString &plain, const QString &rich = QString());
+    void infoMessage(KJob *job, const QString &plain, const QString &rich = QString()); // KF6 TODO remove the 'rich' argument
 
     /**
      * Emitted to display a warning about this job.
@@ -464,6 +486,8 @@ Q_SIGNALS:
 Q_SIGNALS:
     // These signals must be connected from KIO::KCoreDirLister (among others),
     // therefore they must be public.
+
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 80)
     /**
      * Emitted when we know the amount the job will have to process. The unit of this
      * amount is sent too. It can be emitted several times if the job manages several
@@ -475,8 +499,52 @@ Q_SIGNALS:
      * @param job the job that emitted this signal
      * @param unit the unit of the total amount
      * @param amount the total amount
+     *
+     * @deprecated since 5.80, use the KJob::totalAmountChanged(KJob *, KJob::Unit, qulonglong) signal instead
      */
-    void totalAmount(KJob *job, KJob::Unit unit, qulonglong amount);
+    KCOREADDONS_DEPRECATED_VERSION(5, 80, "Use KJob::totalAmountChanged(KJob *, KJob::Unit, qulonglong) signal")
+    void totalAmount(KJob *job, KJob::Unit unit, qulonglong amount); // clazy:exclude=overloaded-signal
+#endif
+
+    /**
+     * Emitted when we know the amount the job will have to process. The unit of this
+     * amount is sent too. It can be emitted several times if the job manages several
+     * different units.
+     *
+     * @note This is a private signal, it shouldn't be emitted directly by subclasses of
+     * KJob, use setTotalAmount() instead.
+     *
+     * @param job the job that emitted this signal
+     * @param unit the unit of the total amount
+     * @param amount the total amount
+     *
+     * @since 5.80
+     */
+    void totalAmountChanged(KJob *job, KJob::Unit unit, qulonglong amount
+    #if !defined(K_DOXYGEN)
+    , QPrivateSignal
+    #endif
+    );
+
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 80)
+    /**
+     * Regularly emitted to show the progress of this job by giving the current amount.
+     * The unit of this amount is sent too. It can be emitted several times if the job
+     * manages several different units.
+     *
+     * @note This is a private signal, it shouldn't be emitted directly by subclasses of
+     * KJob, use setProcessedAmount() instead.
+     *
+     * @param job the job that emitted this signal
+     * @param unit the unit of the processed amount
+     * @param amount the processed amount
+     *
+     * @deprecated since 5.80, use the KJob::processedAmountChanged(KJob *, KJob::Unit, qulonglong)
+     * signal instead
+     */
+    KCOREADDONS_DEPRECATED_VERSION(5, 80, "Use KJob::processedAmountChanged(KJob *, KJob::Unit, qulonglong) signal")
+    void processedAmount(KJob *job, KJob::Unit unit, qulonglong amount); // clazy:exclude=overloaded-signal
+#endif
 
     /**
      * Regularly emitted to show the progress of this job by giving the current amount.
@@ -489,8 +557,14 @@ Q_SIGNALS:
      * @param job the job that emitted this signal
      * @param unit the unit of the processed amount
      * @param amount the processed amount
+     *
+     * @since 5.80
      */
-    void processedAmount(KJob *job, KJob::Unit unit, qulonglong amount);
+    void processedAmountChanged(KJob *job, KJob::Unit unit, qulonglong amount
+    #if !defined(K_DOXYGEN)
+    , QPrivateSignal
+    #endif
+    );
 
     /**
      * Emitted when we know the size of this job (data size in bytes for transfers,
@@ -516,6 +590,7 @@ Q_SIGNALS:
      */
     void processedSize(KJob *job, qulonglong size);
 
+#if KCOREADDONS_ENABLE_DEPRECATED_SINCE(5, 80)
     /**
      * Progress signal showing the overall progress of the job
      * This is valid for any kind of job, and allows using a
@@ -528,8 +603,35 @@ Q_SIGNALS:
      *
      * @param job the job that emitted this signal
      * @param percent the percentage
+     *
+     * @deprecated since 5.80, use the KJob::percentChanged(KJob *, unsigned long)
+     * signal instead.
      */
-    void percent(KJob *job, unsigned long percent);
+    KCOREADDONS_DEPRECATED_VERSION(5, 80, "Use KJob::percentChanged(KJob *, unsigned long) signal")
+    void percent(KJob *job, unsigned long percent); // clazy:exclude=overloaded-signal
+#endif
+
+    /**
+     * Progress signal showing the overall progress of the job. This is
+     * valid for any kind of job, and allows using a progress bar very
+     * easily. (see KProgressBar).
+     *
+     * Note that this signal is not emitted for finished jobs.
+     *
+     * @note This is a private signal, it shouldn't be emitted directly
+     * by subclasses of KJob, use emitPercent(), setPercent() setTotalAmount()
+     * or setProcessedAmount() instead.
+     *
+     * @param job the job that emitted this signal
+     * @param percent the percentage
+     *
+     * @since 5.80
+     */
+    void percentChanged(KJob *job, unsigned long percent
+    #if !defined(K_DOXYGEN)
+    , QPrivateSignal
+    #endif
+    );
 
     /**
      * Emitted to display information about the speed of this job.
@@ -543,6 +645,15 @@ Q_SIGNALS:
     void speed(KJob *job, unsigned long speed);
 
 protected:
+    /**
+     * Returns if the job has been finished and has emitted the finished() signal.
+     *
+     * @return if the job has been finished
+     * @see finished()
+     * @since 5.75
+     */
+    bool isFinished() const;
+
     /**
      * Sets the error code.
      *
@@ -555,7 +666,7 @@ protected:
      * @code
      * enum {
      *   InvalidFoo = UserDefinedError,
-     *   BarNotFound
+     *   BarNotFound,
      * };
      * @endcode
      *
@@ -599,10 +710,22 @@ protected:
      */
     void setTotalAmount(Unit unit, qulonglong amount);
 
+    /**
+     * Sets the unit that will be used internally to calculate
+     * the progress percentage.
+     * The default progress unit is Bytes.
+     * @since 5.76
+     */
+    void setProgressUnit(Unit unit);
+
     /**
      * Sets the overall progress of the job. The percent() signal
      * is emitted if the value changed.
      *
+     * The job takes care of this if you call setProcessedAmount
+     * in Bytes (or the unit set by setProgressUnit).
+     * This method allows you to set your own progress, as an alternative.
+     *
      * @param percentage the new overall progress
      */
     void setPercent(unsigned long percentage);
@@ -639,13 +762,12 @@ protected:
     void emitSpeed(unsigned long speed);
 
 protected:
-    KJobPrivate *const d_ptr;
+    std::unique_ptr<KJobPrivate> const d_ptr;
     KJob(KJobPrivate &dd, QObject *parent);
 
 private:
     void finishJob(bool emitResult);
 
-    Q_PRIVATE_SLOT(d_func(), void _k_speedTimeout())
     Q_DECLARE_PRIVATE(KJob)
 };
 
